// Written by Xavier Wielemans (xavier@tinybigstory.be) - feel free to contact me if you have any issue with the Unity integration. Use at your own risks. Good work!

using UnityEngine;
using System.Runtime.InteropServices;

public class TestPeperoni : MonoBehaviour
{
    [DllImport("usbdmx64")]
    private static extern ushort usbdmx_version();

    [DllImport("usbdmx64")]
    private static extern unsafe bool usbdmx_open(ushort device, void** h);

    [DllImport("usbdmx64")]
    private static extern unsafe bool usbdmx_close(void* h);

    [DllImport("usbdmx64")]
    private static extern unsafe int usbdmx_dmxtx(void* h, byte universe, ushort slots, void* buffer);

    unsafe void* handle = null;
    bool isOpen = false;

    void Start()
    {
        print("Version= 0x" + usbdmx_version().ToString("X"));
        print("Opening DMX...");
        unsafe
        {
            void* h = null;
            var result = usbdmx_open(0, &h);
            print("returned " + (result ? "true" : "false"));
            handle = h;
            isOpen = result;
        }
    }

    byte[] buffer = new byte[512];

    private void Update()
    {
        if (isOpen)
        {
            print("sending 512 random bytes");

            for (int i = 0; i < 512; i++)
            {
                buffer[i] = (byte)Random.Range(0, 256);
            }

            unsafe
            {
                fixed (byte* pBuffer = &buffer[0])
                {
                    var result = usbdmx_dmxtx(handle, 0, 512, pBuffer);
                    print("returned " + result);
                }
            }
        }

        if (Input.GetKeyDown(KeyCode.X))
        {
            print("Sending zeros before closing");
            for (int i = 0; i < 512; i++)
            {
                buffer[i] = 0;
            }

            unsafe
            {
                fixed (byte* pBuffer = &buffer[0])
                {
                    var intResult = usbdmx_dmxtx(handle, 0, 512, pBuffer);
                    print("returned " + intResult);
                }
                print("closing");
                var result = usbdmx_close(handle);
                print("returned " + (result ? "true" : "false"));
                handle = null;
            }
            isOpen = false;
        }
    }
}
